@@ -11,7 +11,7 @@ use warnings;
 
 no warnings qw( threads recursion uninitialized );
 
-our $VERSION = '1.879';
+our $VERSION = '1.900';
 
 ## no critic (BuiltinFunctions::ProhibitStringyEval)
 ## no critic (Subroutines::ProhibitSubroutinePrototypes)
@@ -121,7 +121,7 @@ BEGIN {
          return $self->{$_p};
       };
    }
-   for my $_p (qw( chunk_id task_id task_wid wid )) {
+   for my $_p (qw( chunk_id seed task_id task_wid wid )) {
       *{ $_p } = sub () {
          my $self = shift; $self = $MCE unless ref($self);
          return $self->{"_${_p}"};
@@ -165,6 +165,8 @@ sub import {
       $_p->{TMP_DIR}     = shift, next if ( $_arg eq 'tmp_dir' );
       $_p->{FREEZE}      = shift, next if ( $_arg eq 'freeze' );
       $_p->{THAW}        = shift, next if ( $_arg eq 'thaw' );
+      $_p->{INIT_RELAY}  = shift, next if ( $_arg eq 'init_relay' );
+      $_p->{USE_THREADS} = shift, next if ( $_arg eq 'use_threads' );
 
       if ( $_arg eq 'export_const' || $_arg eq 'const' ) {
          if ( shift eq '1' ) {
@@ -206,7 +208,7 @@ sub import {
 use constant {
 
    # Max data channels. This cannot be greater than 8 on MSWin32.
-   DATA_CHANNELS  => ($^O eq 'MSWin32') ? 8 : 10,
+   DATA_CHANNELS  => 8,
 
    # Max GC size. Undef variable when exceeding size.
    MAX_GC_SIZE    => 1024 * 1024 * 64,
@@ -361,6 +363,10 @@ sub _croak {
    $\ = undef; goto &Carp::croak;
 }
 
+sub _relay (;&) {
+   goto &MCE::relay;
+}
+
 use MCE::Core::Validation ();
 use MCE::Core::Manager ();
 use MCE::Core::Worker ();
@@ -380,6 +386,12 @@ sub new {
    $self{freeze}      ||= $_def->{$_pkg}{FREEZE}      || $_freeze;
    $self{thaw}        ||= $_def->{$_pkg}{THAW}        || $_thaw;
 
+   $self{init_relay} = $_def->{$_pkg}{INIT_RELAY}
+      if (exists $_def->{$_pkg}{INIT_RELAY});
+
+   $self{use_threads} = $_def->{$_pkg}{USE_THREADS}
+      if (exists $_def->{$_pkg}{USE_THREADS});
+
    if (exists $self{_module_instance}) {
       $self{_init_total_workers} = $self{max_workers};
       $self{_chunk_id} = $self{_task_wid} = $self{_wrk_status} = 0;
@@ -477,7 +489,7 @@ sub new {
       ? refaddr($self{input_data}) : 0;
 
    my $_data_channels = ("$$.$_tid" eq $_oid)
-      ? ( $INC{'MCE/Channel.pm'} ? 6 : DATA_CHANNELS )
+      ? ( $INC{'MCE/Channel.pm'} ? 4 : DATA_CHANNELS )
       : 2;
 
    my $_total_workers = 0;
@@ -493,10 +505,13 @@ sub new {
    $self{_data_channels} = ($_total_workers < $_data_channels)
       ? $_total_workers : $_data_channels;
 
-   $self{_lock_chn} = ($_total_workers > $_data_channels) ? 1 : 0;
+   $self{_lock_chn} = ($_total_workers > $self{_data_channels}) ? 1 : 0;
    $self{_lock_chn} = 1 if $INC{'MCE/Child.pm'} || $INC{'MCE/Hobo.pm'};
 
-   $MCE = \%self if ($MCE->{_wid} == 0);
+   if ($MCE->{_wid} == 0) {
+      $MCE = \%self;
+      weaken $MCE if (defined wantarray);
+   }
 
    return \%self;
 }
@@ -573,7 +588,7 @@ sub spawn {
       if ($_is_MSWin32 && defined $TOP_HDLR && $TOP_HDLR->{_spawned}) {
          $TOP_HDLR->shutdown(1);
       }
-      $TOP_HDLR = $self;
+      weaken($TOP_HDLR = $self);
    }
    elsif (refaddr($self) != refaddr($TOP_HDLR)) {
       ## Reduce the maximum number of channels for nested sessions.
@@ -597,7 +612,7 @@ sub spawn {
    my $_max_workers   = _get_max_workers($self);
    my $_use_threads   = $self->{use_threads};
 
-   ## Create locks for data channels.
+   ## Create [ 0 including 1 up to 8 ] locks for data channels (max 9).
    $self->{'_mutex_0'} = MCE::Mutex->new( impl => 'Channel' );
 
    if ($self->{_lock_chn}) {
@@ -605,20 +620,21 @@ sub spawn {
          for (1 .. $_data_channels);
    }
 
-   ## Create sockets for IPC.                             sync, comm, data
+   ## Create two locks for use by MCE::Core::Input::{ Handle or Sequence }.
+   $self->{'_mutex_'.$_} = MCE::Mutex->new( impl => 'Channel' ) for (10 .. 11);
+
+   ## Create sockets for IPC.                      sync, comm, input, data
    MCE::Util::_sock_pair($self, qw(_bsb_r_sock _bsb_w_sock), undef, 1);
    MCE::Util::_sock_pair($self, qw(_com_r_sock _com_w_sock), undef, 1);
-
+   MCE::Util::_sock_pair($self, qw(_que_r_sock _que_w_sock), undef, 1);
    MCE::Util::_sock_pair($self, qw(_dat_r_sock _dat_w_sock), 0);
+
    MCE::Util::_sock_pair($self, qw(_dat_r_sock _dat_w_sock), $_, 1)
       for (1 .. $_data_channels);
 
-   setsockopt($self->{_dat_r_sock}->[0], SOL_SOCKET, SO_RCVBUF, pack('i', 4096))
-      if ($^O ne 'aix' && $^O ne 'linux');
-
-   ($_is_MSWin32)                                                   # input
-      ? MCE::Util::_pipe_pair($self, qw(_que_r_sock _que_w_sock))
-      : MCE::Util::_sock_pair($self, qw(_que_r_sock _que_w_sock), undef, 1);
+   if ($^O !~ /linux|android|aix/) {
+      setsockopt($self->{_dat_r_sock}->[0], SOL_SOCKET, SO_RCVBUF, pack('i', 4096));
+   }
 
    if (defined $self->{init_relay}) {                               # relay
       unless ($INC{'MCE/Relay.pm'}) {
@@ -628,7 +644,7 @@ sub spawn {
          for (0 .. $_max_workers - 1);
    }
 
-   $self->{_seed} = int(rand() * 1e9);
+   $self->{_seed} = int(CORE::rand() * 1e9);
 
    ## -------------------------------------------------------------------------
 
@@ -724,7 +740,7 @@ sub spawn {
    $SIG{__DIE__}  = $_die_handler;
    $SIG{__WARN__} = $_warn_handler;
 
-   $MCE = $self if ($MCE->{_wid} == 0);
+   $MCE = $self if ($MCE->{_wid} == 0 && refaddr($MCE) != refaddr($self));
 
    $MCE::_GMUTEX->unlock() if ($_tid && $MCE::_GMUTEX);
 
@@ -770,7 +786,7 @@ sub process {
 }
 
 sub relay (;&) {
-   _croak('MCE::relay: (init_relay) is not specified')
+   _croak('MCE::relay: (init_relay) is not defined')
       unless (defined $MCE->{init_relay});
 }
 
@@ -803,7 +819,7 @@ sub AUTOLOAD {
    # relay stubs for MCE::Relay
 
    if ($_fcn eq 'relay_lock' || $_fcn eq 'relay_recv') {
-      _croak('MCE::relay: (init_relay) is not specified')
+      _croak('MCE::relay: (init_relay) is not defined')
          unless (defined $MCE->{init_relay});
    }
    elsif ($_fcn eq 'relay_final') {
@@ -981,7 +997,7 @@ sub run {
    local $SIG{__DIE__}  = \&MCE::Signal::_die_handler;
    local $SIG{__WARN__} = \&MCE::Signal::_warn_handler;
 
-   $MCE = $self if ($MCE->{_wid} == 0);
+   $MCE = $self if ($MCE->{_wid} == 0 && refaddr($MCE) != refaddr($self));
 
    my ($_input_data, $_input_file, $_input_glob, $_seq);
    my ($_abort_msg, $_first_msg, $_run_mode, $_single_dim);
@@ -1206,6 +1222,9 @@ sub run {
    if ($_auto_shutdown || $self->{_total_exited}) {
       $self->shutdown();
    }
+   elsif ($INC{'MCE/Simple.pm'}) {
+      $self->shutdown();
+   }
    elsif ($^S || $ENV{'PERL_IPERL_RUNNING'}) {
       if (
          !$INC{'Mojo/IOLoop.pm'} && !$INC{'Win32/GUI.pm'} &&
@@ -1360,18 +1379,15 @@ sub shutdown {
    $_COM_R_SOCK = undef;
 
    MCE::Util::_destroy_socks($self, qw(
-      _bsb_w_sock _bsb_r_sock _com_w_sock _com_r_sock
+      _bsb_w_sock _bsb_r_sock _com_w_sock _com_r_sock _que_w_sock _que_r_sock
       _dat_w_sock _dat_r_sock _rla_w_sock _rla_r_sock
    ));
 
-   ($_is_MSWin32)
-      ? MCE::Util::_destroy_pipes($self, qw( _que_w_sock _que_r_sock ))
-      : MCE::Util::_destroy_socks($self, qw( _que_w_sock _que_r_sock ));
-
    ## -------------------------------------------------------------------------
 
    ## Destroy mutexes.
    for my $_i (0 .. $_data_channels) { delete $self->{'_mutex_'.$_i}; }
+   for my $_j (10 .. 11) { delete $self->{'_mutex_'.$_j}; } # input mutexes
 
    ## Remove session directory.
    rmdir $_sess_dir if (defined $_sess_dir && -d $_sess_dir);
@@ -1578,8 +1594,7 @@ sub sess_dir {
    return $self->{_sess_dir} if defined $self->{_sess_dir};
 
    if ($self->{_wid} == 0) {
-      $self->{_sess_dir} = $self->{_spawned}
-         ? _make_sessdir($self) : undef;
+      $self->{_sess_dir} = $self->{_spawned} ? _make_sessdir($self) : undef;
    }
    else {
       my $_chn        = $self->{_chn};
@@ -1801,7 +1816,7 @@ sub printf {
    }
 
    $_fmt  = shift || '%s';
-   $_data = sprintf($_fmt, scalar @_ ? @_ : $_);
+   $_data = _sprintf($_fmt, scalar @_ ? @_ : $_);
 
    return _do_send_glob($self, $_glob, $_fd, \$_data) if $_fd;
    return _do_send($self, SENDTO_STDOUT, undef, \$_data) if $self->{_wid};
@@ -1843,7 +1858,9 @@ sub say {
 
 sub _exit {
    my $self = shift;
+   my $_has_guard = (exists $self->{_guard} && $self->{_guard}->[0]) ? 1 : 0;
 
+   @{ $self->{_guard} } = () if $_has_guard;
    delete $self->{_wuf}; _end();
 
    ## Exit thread/child process.
@@ -1863,7 +1880,7 @@ sub _exit {
       };
    }
 
-   if ($self->{posix_exit} && !$_is_MSWin32) {
+   if ($self->{posix_exit} && !$_has_guard && !$_is_MSWin32) {
       eval { MCE::Mutex::Channel::_destroy() };
       POSIX::_exit(0) if $INC{'POSIX.pm'};
       CORE::kill('KILL', $$);
@@ -1916,11 +1933,11 @@ sub _make_sessdir {
 }
 
 sub _sprintf {
-   my ($_fmt, $_arg) = @_;
+   my $_fmt = shift;
    # remove tainted'ness
-   ($_fmt) = $_fmt =~ /(.*)/;
+   ($_fmt) = $_fmt =~ /(.*)/s;
 
-   return sprintf("$_fmt", $_arg);
+   return sprintf("$_fmt", @_);
 }
 
 sub _sync_buffer_to_array {
@@ -1995,24 +2012,33 @@ sub _dispatch {
    $self->{_is_thread} = $_is_thread;
    $self->{_pid}       = $_is_thread ? $$ .'.'. threads->tid() : $$;
 
-   ## Sets the seed of the base generator uniquely between workers.
-   ## The new seed is computed using the current seed and $_wid value.
-   ## One may set the seed at the application level for predictable
-   ## results (non-thread workers only). Ditto for Math::Prime::Util,
-   ## Math::Random, and Math::Random::MT::Auto.
+   if (!$self->{use_threads}) {
+      MCE::Child->_clear() if $INC{'MCE/Child.pm'};
+      MCE::Hobo->_clear() if $INC{'MCE/Hobo.pm'};
+   }
+
+   # Set the seed of the base generator uniquely between workers.
+   # The new seed is computed using the current seed and ID value.
+   # One may set the seed at the application level for predictable
+   # results (non-thread workers only). Ditto for Math::Prime::Util,
+   # Math::Random, Math::Random::MT::Auto, and PDL.
+   #
+   # MCE 1.892, 2024-06-08: Enable predictability running threads.
+   # Output matches non-threads for CORE, Math::Prime::Util, and
+   # Math::Random::MT::Auto. https://perlmonks.org/?node_id=11159834
 
    {
-      my ($_wid, $_seed) = ($_args[1], $self->{_seed});
-      srand(abs($_seed - ($_wid * 100000)) % 2147483560);
+      my $_wid  = $_args[1];
+      my $_seed = abs($self->{_seed} - ($_wid * 100000)) % 2147483560;
 
+      CORE::srand($_seed) if (!$self->{use_threads} || $] ge '5.020000'); # drand48
+      Math::Prime::Util::srand($_seed) if $INC{'Math/Prime/Util.pm'};
+
+      # [etj] identified a race condition in PDL running threads
+      # https://perlmonks.org/?node_id=11159841
       if (!$self->{use_threads}) {
-         Math::Prime::Util::srand(abs($_seed - ($_wid * 100000)) % 2147483560)
-            if ( $INC{'Math/Prime/Util.pm'} );
-
-         MCE::Hobo->_clear()
-            if ( $INC{'MCE/Hobo.pm'} && MCE::Hobo->can('_clear') );
-
-         MCE::Child->_clear() if $INC{'MCE/Child.pm'};
+         PDL::srand($_seed) if $INC{'PDL.pm'} && PDL->can('srand'); # PDL 2.062 ~ 2.089
+         PDL::srandom($_seed) if $INC{'PDL.pm'} && PDL->can('srandom'); # PDL 2.089_01+
       }
    }
 
@@ -2026,7 +2052,7 @@ sub _dispatch {
       Math::Random::random_set_seed($_new_seed, $_new_seed);
    }
 
-   if (!$self->{use_threads} && $INC{'Math/Random/MT/Auto.pm'}) {
+   if ($INC{'Math/Random/MT/Auto.pm'}) {
       my ($_wid, $_cur_seed) = (
          $_args[1], Math::Random::MT::Auto::get_seed()->[0]
       );
