@@ -11,7 +11,7 @@ use warnings;
 
 no warnings qw( uninitialized once );
 
-our $VERSION = '1.879';
+our $VERSION = '1.900';
 
 ## no critic (BuiltinFunctions::ProhibitStringyEval)
 ## no critic (TestingAndDebugging::ProhibitNoStrict)
@@ -48,7 +48,6 @@ BEGIN {
 
 use MCE::Util ();
 
-my $is_MSWin32 = ( $^O eq 'MSWin32' ) ? 1 : 0;
 my $tid = $INC{'threads.pm'} ? threads->tid() : 0;
 
 sub new {
@@ -56,7 +55,7 @@ sub new {
    my $impl = defined( $argv{impl} ) ? ucfirst( lc $argv{impl} ) : 'Mutex';
 
    # Replace 'fast' with 'Fast' in the implementation value.
-   $impl =~ s/fast$/Fast/;
+   $impl =~ s/fast/Fast/;
 
    $impl = 'Threads'     if ( $impl eq 'Mutex' && $^O eq 'MSWin32' );
    $impl = 'ThreadsFast' if ( $impl eq 'MutexFast' && $^O eq 'MSWin32' );
@@ -132,7 +131,7 @@ MCE::Channel - Queue-like and two-way communication capability
 
 =head1 VERSION
 
-This document describes MCE::Channel version 1.879
+This document describes MCE::Channel version 1.900
 
 =head1 SYNOPSIS
 
@@ -736,7 +735,7 @@ Mario E. Roy, S<E<lt>marioeroy AT gmail DOT comE<gt>>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2019-2022 by Mario E. Roy
+Copyright (C) 2019-2024 by Mario E. Roy
 
 MCE::Channel is released under the same license as Perl.
 
