@@ -11,7 +11,7 @@ use warnings;
 
 no warnings qw( uninitialized once );
 
-our $VERSION = '1.879';
+our $VERSION = '1.900';
 
 use threads;
 use threads::shared;
@@ -229,20 +229,15 @@ sub send2 {
    my $data = $freeze->([ @_ ]);
 
    local $\ = undef if (defined $\);
-   local $MCE::Signal::SIG;
 
    {
       my $c_sock = $self->{c2_sock} || $self->{c_sock};
 
-      local $MCE::Signal::IPC = 1;
       CORE::lock $self->{cw_mutex};
-
       MCE::Util::_sock_ready_w( $c_sock ) if $is_MSWin32;
       print { $c_sock } pack('i', length $data), $data;
    }
 
-   CORE::kill($MCE::Signal::SIG, $$) if $MCE::Signal::SIG;
-
    return 1;
 }
 
@@ -320,7 +315,7 @@ MCE::Channel::Threads - Channel for producer(s) and many consumers
 
 =head1 VERSION
 
-This document describes MCE::Channel::Threads version 1.879
+This document describes MCE::Channel::Threads version 1.900
 
 =head1 DESCRIPTION
 
@@ -339,7 +334,7 @@ The API is described in L<MCE::Channel>.
  my $chnl_a = MCE::Channel->new( impl => 'Threads' );
 
  # Specify the 'mp' option for safe use by two or more producers
- # sending or recieving on the left side of the channel (i.e.
+ # sending or receiving on the left side of the channel (i.e.
  # ->enqueue/->send or ->recv2/->recv2_nb).
 
  my $chnl_b = MCE::Channel->new( impl => 'Threads', mp => 1 );
