@@ -11,7 +11,7 @@ no warnings qw( threads recursion uninitialized once redefine );
 
 package MCE::Child;
 
-our $VERSION = '1.889';
+our $VERSION = '1.900';
 
 ## no critic (BuiltinFunctions::ProhibitStringyEval)
 ## no critic (Subroutines::ProhibitExplicitReturnUndef)
@@ -49,6 +49,7 @@ my ( $_MNGD, $_DATA, $_DELY, $_LIST ) = ( {}, {}, {}, {} );
 
 my $_is_MSWin32 = ( $^O eq 'MSWin32' ) ? 1 : 0;
 my $_tid        = ( $INC{'threads.pm'} ) ? threads->tid() : 0;
+my $_yield_secs = ( $^O =~ /mswin|mingw|msys|cygwin/i ) ? 0.015 : 0.008;
 
 sub CLONE {
    $_tid = threads->tid(), &_clear() if $INC{'threads.pm'};
@@ -79,6 +80,17 @@ sub _max_workers {
 
 bless my $_SELF = { MGR_ID => "$$.$_tid", WRK_ID => $$ }, __PACKAGE__;
 
+sub MCE::Child::_guard::DESTROY {
+   my ($pkg, $id) = @{ $_[0] };
+
+   if (defined $pkg && $id eq "$$.$_tid") {
+      @{ $_[0] } = ();
+      MCE::Child->finish($pkg);
+   }
+
+   return;
+}
+
 sub init {
    shift if ( defined $_[0] && $_[0] eq __PACKAGE__ );
 
@@ -86,17 +98,18 @@ sub init {
    # max_workers child_timeout posix_exit on_start on_finish void_context
    # ---------------------------------------------------------------------
 
-   my $pkg = "$$.$_tid.".( caller eq __PACKAGE__ ? caller(1) : caller );
-   my $mngd = $_MNGD->{$pkg} = ( ref $_[0] eq 'HASH' ) ? shift : { @_ };
+   my $opt = ( ref $_[0] eq 'HASH' ) ? shift : { @_ };
+   my $pkg = "$$.$_tid.".( delete $opt->{caller} || caller() );
+   my $mngd = $_MNGD->{$pkg} = $opt;
 
    @_ = ();
 
    $mngd->{MGR_ID} = "$$.$_tid", $mngd->{PKG} = $pkg,
    $mngd->{WRK_ID} =  $$;
 
-   &_force_reap($pkg), $_DATA->{$pkg}->clear() if ( exists $_LIST->{$pkg} );
+   &_force_reap($pkg), $_DATA->{$pkg}->clear() if ( defined $_LIST->{$pkg} );
 
-   if ( !exists $_LIST->{$pkg} ) {
+   if ( !defined $_LIST->{$pkg} ) {
       $MCE::_GMUTEX->lock() if ( $_tid && $MCE::_GMUTEX );
       sleep 0.015 if $_tid;
 
@@ -107,8 +120,9 @@ sub init {
       $_LIST->{ $pkg } = MCE::Child::_ordhash->new();
       $_DELY->{ $pkg } = MCE::Child::_delay->new( $chnl );
       $_DATA->{ $pkg } = MCE::Child::_hash->new( $chnl );
-      $_DATA->{"$pkg:seed"} = int(rand() * 1e9);
-      $_DATA->{"$pkg:id"  } = 0;
+      $_DATA->{"$pkg:id"} = 0;
+
+      $_DATA->{"$pkg:seed"} = int(CORE::rand() * 1e9);
 
       $MCE::_GMUTEX->unlock() if ( $_tid && $MCE::_GMUTEX );
    }
@@ -134,7 +148,9 @@ sub init {
    require POSIX
       if ( $mngd->{on_finish} && !$INC{'POSIX.pm'} && !$_is_MSWin32 );
 
-   return;
+   defined wantarray
+      ? bless([$pkg, "$$.$_tid"], MCE::Child::_guard::)
+      : ();
 }
 
 ###############################################################################
@@ -155,9 +171,11 @@ sub mce_child (&;@) {
 }
 
 sub create {
-   my $mngd = $_MNGD->{ "$$.$_tid.".caller() } || do {
+   my $caller = caller();
+
+   my $mngd = $_MNGD->{ "$$.$_tid.$caller" } || do {
       # construct mngd internally on first use unless defined
-      init(); $_MNGD->{ "$$.$_tid.".caller() };
+      init( caller => $caller ); $_MNGD->{ "$$.$_tid.$caller" };
    };
 
    shift if ( $_[0] eq __PACKAGE__ );
@@ -170,7 +188,7 @@ sub create {
    $self->{MGR_ID} = $mngd->{MGR_ID}, $self->{PKG} = $mngd->{PKG};
    $self->{ident } = shift if ( !ref $_[0] && ref $_[1] eq 'CODE' );
 
-   my $func = shift; $func = caller().'::'.$func
+   my $func = shift; $func = $caller.'::'.$func
       if ( !ref $func && length $func && index($func,':') < 0 );
 
    if ( !defined $func ) {
@@ -184,21 +202,23 @@ sub create {
 
    $_DATA->{"$pkg:id"} = 10000 if ( ( my $id = ++$_DATA->{"$pkg:id"} ) >= 2e9 );
 
+   # Reap completed child processes.
    {
-      # Reap completed child processes.
-      local ($SIG{CHLD}, $!, $?, $_); map {
-         waitpid($_, 0); _reap_child($list->del($_), 0); ();
+      local ($SIG{CHLD}, $!, $?, $_);
+      map {
+         $_ = substr($_, 1); # strip leading 'R'
+         my $child = $list->del($_);
+         if ( ! $child->{REAPED} ) {
+             waitpid($child->{WRK_ID}, 0);
+             _reap_child($child, 0);
+         }
+         ();
       }
-      $_DATA->{$pkg}->reap_data;
+      $_DATA->{$pkg}->get_done();
    }
 
-   if ( $max_workers || $self->{IGNORE} ) {
-      # Wait for a slot if saturated.
-      if ( $max_workers && $list->len() >= $max_workers ) {
-         my $count = $list->len() - $max_workers + 1;
-         _wait_one($pkg) for 1 .. $count;
-      }
-   }
+   # Wait for a slot if saturated.
+   _wait_one($pkg) if ( $max_workers && $list->len() >= $max_workers );
 
    # ~~~ ~~~ ~~~ ~~~ ~~~ ~~~ ~~~ ~~~ ~~~ ~~~ ~~~ ~~~ ~~~ ~~~ ~~~ ~~~ ~~~ ~~~
 
@@ -236,18 +256,22 @@ sub create {
          $_DATA->{ $_SELF->{PKG} }->set('S'.$$, '') unless $self->{IGNORE};
          CORE::kill($killed, $$) if $killed;
 
-         # Sets the seed of the base generator uniquely between workers.
+         MCE::Child->_clear() if $INC{'MCE/Child.pm'};
+         MCE::Hobo->_clear() if $INC{'MCE/Hobo.pm'};
+
+         # Set the seed of the base generator uniquely between workers.
          # The new seed is computed using the current seed and ID value.
          # One may set the seed at the application level for predictable
-         # results. Ditto for Math::Prime::Util, Math::Random, and
+         # results. Ditto for PDL, Math::Prime::Util, Math::Random, and
          # Math::Random::MT::Auto.
 
-         srand( abs($_DATA->{"$pkg:seed"} - ($id * 100000)) % 2147483560 );
+         {
+            my $seed = abs($_DATA->{"$pkg:seed"} - ($id * 100000)) % 2147483560;
 
-         if ( $INC{'Math/Prime/Util.pm'} ) {
-            Math::Prime::Util::srand(
-                abs($_DATA->{"$pkg:seed"} - ($id * 100000)) % 2147483560
-            );
+            CORE::srand($seed);
+            PDL::srand($seed) if $INC{'PDL.pm'} && PDL->can('srand'); # PDL 2.062 ~ 2.089
+            PDL::srandom($seed) if $INC{'PDL.pm'} && PDL->can('srandom'); # PDL 2.089_01+
+            Math::Prime::Util::srand($seed) if $INC{'Math/Prime/Util.pm'};
          }
 
          if ( $INC{'Math/Random.pm'} ) {
@@ -307,15 +331,16 @@ sub exit {
    }
    elsif ( $wrk_id == $$ ) {
       alarm 0; my ( $exit_status, @res ) = @_; $? = $exit_status || 0;
-      $_DATA->{$pkg}->set('R'.$wrk_id, @res ? \@res : '');
+      $_DATA->{$pkg}->set('R'.$wrk_id, @res ? \@res : '')
+         unless $self->{IGNORE};
       die "Child exited ($?)\n";
       _exit($?); # not reached
    }
 
    return $self if $self->{REAPED};
 
-   if ( exists $_DATA->{$pkg} ) {
-      sleep 0.015 until $_DATA->{$pkg}->exists('S'.$wrk_id);
+   if ( defined $_DATA->{$pkg} ) {
+      sleep $_yield_secs until $_DATA->{$pkg}->exists('S'.$wrk_id);
    } else {
       sleep 0.030;
    }
@@ -333,12 +358,12 @@ sub finish {
    _croak('Usage: MCE::Child->finish()') if ref($_[0]);
    shift if ( defined $_[0] && $_[0] eq __PACKAGE__ );
 
-   my $pkg = defined($_[0]) ? $_[0] : caller();
+   my $pkg = defined($_[0]) ? shift : "$$.$_tid.".caller();
 
    if ( $pkg eq 'MCE' ) {
       for my $key ( keys %{ $_LIST } ) { MCE::Child->finish($key); }
    }
-   elsif ( exists $_LIST->{$pkg} ) {
+   elsif ( defined $_LIST->{$pkg} ) {
       return if $MCE::Signal::KILLED;
 
       if ( exists $_DELY->{$pkg} ) {
@@ -402,7 +427,7 @@ sub join {
 
    if ( $self->{REAPED} ) {
       _croak('Child already joined') unless exists( $self->{RESULT} );
-      $_LIST->{$pkg}->del($wrk_id) if ( exists $_LIST->{$pkg} );
+      $_LIST->{$pkg}->del($wrk_id) if ( defined $_LIST->{$pkg} );
 
       return ( defined wantarray )
          ? wantarray ? @{ delete $self->{RESULT} } : delete( $self->{RESULT} )->[-1]
@@ -440,8 +465,8 @@ sub kill {
    }
    if ( $self->{MGR_ID} eq "$$.$_tid" ) {
       return $self if $self->{REAPED};
-      if ( exists $_DATA->{$pkg} ) {
-         sleep 0.015 until $_DATA->{$pkg}->exists('S'.$wrk_id);
+      if ( defined $_DATA->{$pkg} ) {
+         sleep $_yield_secs until $_DATA->{$pkg}->exists('S'.$wrk_id);
       } else {
          sleep 0.030;
       }
@@ -456,14 +481,14 @@ sub list {
    _croak('Usage: MCE::Child->list()') if ref($_[0]);
    my $pkg = "$$.$_tid.".caller();
 
-   ( exists $_LIST->{$pkg} ) ? $_LIST->{$pkg}->vals() : ();
+   ( defined $_LIST->{$pkg} ) ? $_LIST->{$pkg}->vals() : ();
 }
 
 sub list_pids {
    _croak('Usage: MCE::Child->list_pids()') if ref($_[0]);
    my $pkg = "$$.$_tid.".caller(); local $_;
 
-   ( exists $_LIST->{$pkg} ) ? map { $_->pid } $_LIST->{$pkg}->vals() : ();
+   ( defined $_LIST->{$pkg} ) ? map { $_->pid } $_LIST->{$pkg}->vals() : ();
 }
 
 sub list_joinable {
@@ -471,9 +496,7 @@ sub list_joinable {
    my $pkg = "$$.$_tid.".caller();
 
    return () unless ( my $list = $_LIST->{$pkg} );
-   local ($!, $?, $_);
-
-   $_DATA->{$pkg}->reap_data;
+   local ($!, $?, $_); $_DATA->{$pkg}->reap_data;
 
    map {
       ( waitpid($_->{WRK_ID}, _WNOHANG) == 0 ) ? () : do {
@@ -489,9 +512,7 @@ sub list_running {
    my $pkg = "$$.$_tid.".caller();
 
    return () unless ( my $list = $_LIST->{$pkg} );
-   local ($!, $?, $_);
-
-   $_DATA->{$pkg}->reap_data;
+   local ($!, $?, $_); $_DATA->{$pkg}->reap_data;
 
    map {
       ( waitpid($_->{WRK_ID}, _WNOHANG) == 0 ) ? $_ : do {
@@ -518,7 +539,7 @@ sub pending {
    _croak('Usage: MCE::Child->pending()') if ref($_[0]);
    my $pkg = "$$.$_tid.".caller();
 
-   ( exists $_LIST->{$pkg} ) ? $_LIST->{$pkg}->len() : 0;
+   ( defined $_LIST->{$pkg} ) ? $_LIST->{$pkg}->len() : 0;
 }
 
 sub pid {
@@ -533,6 +554,13 @@ sub result {
    wantarray ? @{ delete $self->{RESULT} } : delete( $self->{RESULT} )->[-1];
 }
 
+sub seed {
+   _croak('Usage: MCE::Child->seed()') if ref($_[0]);
+   my $pkg = exists $_SELF->{PKG} ? $_SELF->{PKG} : "$$.$_tid.".caller();
+
+   return $_DATA->{"$pkg:seed"};
+}
+
 sub self {
    ref($_[0]) ? $_[0] : $_SELF;
 }
@@ -542,7 +570,7 @@ sub wait_all {
    my $pkg = "$$.$_tid.".caller();
 
    return wantarray ? () : 0
-      if ( !exists $_LIST->{$pkg} || !$_LIST->{$pkg}->len() );
+      if ( !defined $_LIST->{$pkg} || !$_LIST->{$pkg}->len() );
 
    local $_; ( wantarray )
       ? map { $_->join(); $_ } $_LIST->{$pkg}->vals()
@@ -556,7 +584,7 @@ sub wait_one {
    my $pkg = "$$.$_tid.".caller();
 
    return undef
-      if ( !exists $_LIST->{$pkg} || !$_LIST->{$pkg}->len() );
+      if ( !defined $_LIST->{$pkg} || !$_LIST->{$pkg}->len() );
 
    _wait_one($pkg);
 }
@@ -670,11 +698,12 @@ sub _exit {
    $SIG{__WARN__} = sub {};
 
    threads->exit($exit_status) if ( $INC{'threads.pm'} && $_is_MSWin32 );
+   CORE::kill('KILL', $$) if ( $_SELF->{SIGNALED} && !$_is_MSWin32 );
 
    my $posix_exit = ( exists $_SELF->{posix_exit} )
       ? $_SELF->{posix_exit} : $_MNGD->{ $_SELF->{PKG} }{posix_exit};
 
-   if ( $posix_exit && !$_SELF->{SIGNALED} && !$_is_MSWin32 ) {
+   if ( $posix_exit && !$_is_MSWin32 ) {
       eval { MCE::Mutex::Channel::_destroy() };
       POSIX::_exit($exit_status) if $INC{'POSIX.pm'};
       CORE::kill('KILL', $$);
@@ -685,13 +714,13 @@ sub _exit {
 
 sub _force_reap {
    my ( $count, $pkg ) = ( 0, @_ );
-   return unless ( exists $_LIST->{$pkg} && $_LIST->{$pkg}->len() );
+   return unless ( defined $_LIST->{$pkg} && $_LIST->{$pkg}->len() );
 
    for my $child ( $_LIST->{$pkg}->vals() ) {
       next if $child->{IGNORE};
 
       if ( $child->is_running() ) {
-         sleep(0.015), CORE::kill('KILL', $child->pid())
+         sleep($_yield_secs), CORE::kill('KILL', $child->pid())
             if CORE::kill('ZERO', $child->pid());
          $count++;
       }
@@ -724,9 +753,9 @@ sub _quit {
 
 sub _reap_child {
    my ( $child, $wait_flag ) = @_;
-   return unless $child;
+   return if ( !$child || !defined $child->{PKG} );
 
-   local @_ = $_DATA->{ $child->{PKG} }->get( $child->{WRK_ID}, $wait_flag );
+   local @_ = $_DATA->{ $child->{PKG} }->get($child->{WRK_ID}, $wait_flag);
 
    ( $child->{ERROR}, $child->{RESULT}, $child->{REAPED} ) =
       ( pop || '', length $_[0] ? pop : [], 1 );
@@ -791,7 +820,7 @@ sub _wait_one {
          $self = $list->del($wrk_id), last if waitpid($wrk_id, _WNOHANG);
       }
       last if $self;
-      sleep 0.030;
+      sleep $_yield_secs;
    }
 
    _reap_child($self, 0);
@@ -856,36 +885,36 @@ use constant {
 
 sub new {
    my ( $class, $chnl ) = @_;
-
    bless [ {}, $chnl ], shift;
 }
 
 sub clear {
    my ( $self ) = @_;
-
    1 while ( $self->[1]->recv2_nb() );
-
    %{ $self->[0] } = ();
 }
 
 sub exists {
    my ( $self, $key ) = @_;
+   $self->reap_data;
+   CORE::exists $self->[0]{ $key };
+}
 
-   while ( my $data = $self->[1]->recv2_nb() ) {
-      $self->[0]{ $data->[0] } = $data->[1];
+sub get_done {
+   my ( $self ) = @_;
+   my @ret;
+
+   $self->reap_data;
+   for my $key (keys %{ $self->[0] }) {
+      push @ret, $key if ( substr($key, 0, 1) eq 'R' );
    }
 
-   CORE::exists $self->[0]{ $key };
+   return @ret;
 }
 
 sub get {
    my ( $self, $wrk_id, $wait_flag ) = @_;
-
-   if ( !CORE::exists $self->[0]{ 'R'.$wrk_id } ) {
-      while ( my $data = $self->[1]->recv2_nb() ) {
-         $self->[0]{ $data->[0] } = $data->[1];
-      }
-   }
+   $self->reap_data if ( !CORE::exists $self->[0]{ 'R'.$wrk_id } );
 
    if ( $wait_flag ) {
       local $!;
@@ -899,11 +928,7 @@ sub get {
             $self->[0]{ $data->[0] } = $data->[1];
             waitpid($wrk_id, 0), last if $data->[0] eq 'R'.$wrk_id;
          }
-         if ( !CORE::exists $self->[0]{ 'R'.$wrk_id } ) {
-            while ( my $data = $self->[1]->recv2_nb() ) {
-               $self->[0]{ $data->[0] } = $data->[1];
-            }
-         }
+         $self->reap_data if ( !CORE::exists $self->[0]{ 'R'.$wrk_id } );
       };
    }
 
@@ -919,15 +944,6 @@ sub get {
 sub reap_data {
    my ( $self ) = @_;
 
-   if (wantarray) {
-      my @ret;
-      while ( my $data = $self->[1]->recv2_nb() ) {
-         push @ret, substr($data->[0], 1) if substr($data->[0], 0, 1) eq 'R';
-         $self->[0]{ $data->[0] } = $data->[1];
-      }
-      return @ret;
-   }
-
    while ( my $data = $self->[1]->recv2_nb() ) {
       $self->[0]{ $data->[0] } = $data->[1];
    }
@@ -1007,7 +1023,7 @@ MCE::Child - A threads-like parallelization module compatible with Perl 5.8
 
 =head1 VERSION
 
-This document describes MCE::Child version 1.889
+This document describes MCE::Child version 1.900
 
 =head1 SYNOPSIS
 
@@ -1293,7 +1309,10 @@ processes not yet joined.
 
 The init function accepts a list of MCE::Child options.
 
- MCE::Child->init(
+In scalar context (API available since 1.897), call C<MCE::Child->finish>
+automatically upon leaving the scope or program.
+
+ my $guard = MCE::Child->init(
      max_workers => 'auto',   # default undef, unlimited
 
      # Specify a percentage. MCE::Child 1.876+.
@@ -1530,6 +1549,13 @@ C<wantarray> aware.
  my @res2 = $child2->result();  # ( foo )
  my $res2 = $child2->result();  #   foo
 
+=item MCE::Child->seed()
+
+Class method that returns the internal random generated seed or undefined.
+The seed is generated once during init or initial create.
+
+Current API available since 1.895.
+
 =item MCE::Child->self()
 
 Class method that allows a child to obtain it's own I<MCE::Child> object.
