@@ -11,7 +11,7 @@ use warnings;
 
 no warnings qw( threads recursion uninitialized );
 
-our $VERSION = '1.877';
+our $VERSION = '1.900';
 
 ## no critic (BuiltinFunctions::ProhibitStringyEval)
 ## no critic (Subroutines::ProhibitSubroutinePrototypes)
@@ -47,11 +47,12 @@ sub import {
    };
 
    ## Import functions.
-   no strict 'refs'; no warnings 'redefine';
-
-   *{ $_pkg.'::mce_flow_f' } = \&run_file;
-   *{ $_pkg.'::mce_flow_s' } = \&run_seq;
-   *{ $_pkg.'::mce_flow'   } = \&run;
+   if ($_pkg !~ /^MCE::/) {
+      no strict 'refs'; no warnings 'redefine';
+      *{ $_pkg.'::mce_flow_f' } = \&run_file;
+      *{ $_pkg.'::mce_flow_s' } = \&run_seq;
+      *{ $_pkg.'::mce_flow'   } = \&run;
+   }
 
    ## Process module arguments.
    while ( my $_argument = shift ) {
@@ -62,6 +63,8 @@ sub import {
       $_p->{TMP_DIR}     = shift, next if ( $_arg eq 'tmp_dir' );
       $_p->{FREEZE}      = shift, next if ( $_arg eq 'freeze' );
       $_p->{THAW}        = shift, next if ( $_arg eq 'thaw' );
+      $_p->{INIT_RELAY}  = shift, next if ( $_arg eq 'init_relay' );
+      $_p->{USE_THREADS} = shift, next if ( $_arg eq 'use_threads' );
 
       ## Sereal 3.015+, if available, is used automatically by MCE 1.8+.
       if ( $_arg eq 'sereal' ) {
@@ -91,6 +94,17 @@ sub import {
 ##
 ###############################################################################
 
+sub MCE::Flow::_guard::DESTROY {
+   my ($_pkg, $_id) = @{ $_[0] };
+
+   if (defined $_pkg && $_id eq "$$.$_tid") {
+      @{ $_[0] } = ();
+      MCE::Flow->finish($_pkg);
+   }
+
+   return;
+}
+
 sub init (@) {
 
    shift if (defined $_[0] && $_[0] eq 'MCE::Flow');
@@ -100,7 +114,9 @@ sub init (@) {
 
    @_ = ();
 
-   return;
+   defined wantarray
+      ? bless([$_pkg, "$$.$_tid"], MCE::Flow::_guard::)
+      : ();
 }
 
 sub finish (@) {
@@ -205,17 +221,17 @@ sub run_seq (@) {
          $_pos = $_i;
 
          if ($_r eq '' || $_r =~ /^Math::/) {
-            $_begin = $_[$_pos]; $_end = $_[$_pos + 1];
+            $_begin = $_[$_pos], $_end = $_[$_pos + 1];
             $_params->{$_pid}{sequence} = [
                $_[$_pos], $_[$_pos + 1], $_[$_pos + 2], $_[$_pos + 3]
             ];
          }
          elsif ($_r eq 'HASH') {
-            $_begin = $_[$_pos]->{begin}; $_end = $_[$_pos]->{end};
+            $_begin = $_[$_pos]->{begin}, $_end = $_[$_pos]->{end};
             $_params->{$_pid}{sequence} = $_[$_pos];
          }
          elsif ($_r eq 'ARRAY') {
-            $_begin = $_[$_pos]->[0]; $_end = $_[$_pos]->[1];
+            $_begin = $_[$_pos]->[0], $_end = $_[$_pos]->[1];
             $_params->{$_pid}{sequence} = $_[$_pos];
          }
 
@@ -376,7 +392,7 @@ sub run (@) {
          }
       }
 
-      for my $_k (qw/ tmp_dir freeze thaw /) {
+      for my $_k (qw/ tmp_dir freeze thaw init_relay use_threads /) {
          $_opts{$_k} = $_def->{$_pkg}{uc($_k)}
             if (exists $_def->{$_pkg}{uc($_k)} && !exists $_opts{$_k});
       }
@@ -479,7 +495,7 @@ MCE::Flow - Parallel flow model for building creative applications
 
 =head1 VERSION
 
-This document describes MCE::Flow version 1.877
+This document describes MCE::Flow version 1.900
 
 =head1 DESCRIPTION
 
@@ -787,7 +803,9 @@ The following list options which may be overridden when loading the module.
      chunk_size => 500,               # Default 'auto'
      tmp_dir => "/path/to/app/tmp",   # $MCE::Signal::tmp_dir
      freeze => \&encode_sereal,       # \&Storable::freeze
-     thaw => \&decode_sereal          # \&Storable::thaw
+     thaw => \&decode_sereal,         # \&Storable::thaw
+     init_relay => 0,                 # Default undef; MCE 1.882+
+     use_threads => 0,                # Default undef; MCE 1.882+
  ;
 
 From MCE 1.8 onwards, Sereal 3.015+ is loaded automatically if available.
@@ -806,12 +824,14 @@ Specify C<< Sereal => 0 >> to use Storable instead.
 =back
 
 The init function accepts a hash of MCE options. Unlike with MCE::Stream,
-both gather and bounds_only options may be specified when calling init
-(not shown below).
+both gather and bounds_only options may be specified (not shown below).
+
+In scalar context (API available since 1.897), call C<MCE::Flow->finish>
+automatically upon leaving the scope or program.
 
  use MCE::Flow;
 
- MCE::Flow->init(
+ my $guard = MCE::Flow->init(
     chunk_size => 1, max_workers => 4,
 
     user_begin => sub {
